<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.2 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Cookie.php,v 1.2 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget
 */
require_once 'Gadget.php';

/**
 * Gadget_Cookie
 * 
 * @package Gadget
 * @since   1.2
 */
class Gadget_Cookie
{
    /**
     * Set Cookie
     * 
     * This is a convinience method to set an persistent or a session cookie.  
     * Its lifetime is the date of MAXINT (as unix timestamp) if the cookie is
     * persistent, or to 0 if the cookie is supposed to
     * expire at the end of the session.  The parameters will be urlencoded 
     * just like a query string of an URL.  If the $params argument is not 
     * an array it will be passed through to setcookie().
     * 
     * @see Gadget_Cookie::get()
     * 
     * @static
     * @access  public
     * @return  bool
     * @param   string  $name Cookie name.
     * @param   array   $params Cookie's contents as associative array.
     * @param   bool    $session Whether Cookie is valid for the session only.
     */
    function set($name, $params = array(), $session = false)
    {
        $value = '';
        if (is_array($params)) {
            foreach ($params as $n => $v) {
                $value .= '&'. urlencode($n) .'='. urlencode($v);
            }
        } else {
            $value = $params;
        }
        return setCookie(   $name, $value, $session ? 0 : 2147483647, '/', 
                            Gadget::getConf('Gadget', 'wwwdomain'));
    }
    
    /**
     * Get Cookie
     * 
     * Retrieves the value of an HTTP cookie.  The value will be urldecoded
     * like a query string of an URL unless $parse is FALSE.
     * 
     * @see Gadget_Cookie::set()
     * 
     * @static
     * @access  public
     * @return  mixed
     * @param   string  $name Cookie name.
     * @param   bool    $parse Whether to parse the value of the cookie like
     *                  a query string of an URL.
     */
    function get($name, $parse = true)
    {
        if (!isset($_COOKIE[$name])) {
            return null;
        }
        if (!$parse) {
            return $_COOKIE[$name];
        }
        parse_str($_COOKIE[$name], $params);
        return $params;
    }
    
    /**
     * Reset Cookie
     * 
     * Unsets an HTTP cookie.  The contents of the cookie $name will be set
     * to null and the date to expire will be set to 1 (unix timestamp).
     * 
     * @see Gadget_Cookie::set()
     * 
     * @static
     * @access  public
     * @return  bool
     * @param   string  $name Cookie name.
     */
    function reset($name)
    {
        unset($_COOKIE[$name]);
        return setCookie($name, null, 1, '/', 
            Gadget::getConf('Gadget', 'wwwdomain'));
    }
}
?>