<?php

/**
 * Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.3 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Log.php,v 1.3 2007-03-30 11:40:08 mike Exp $
 */

/** 
 * Gadget_Service_Log
 *
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Log extends Gadget_Service
{
    var $service = SERVICE_LOG;
    var $log;
    
    /**
     * Init
     * 
     * @access  public
     * @return  void
     */
    function init()
    {
        require_once 'Log.php';
        $this->log = Log::factory(
            Gadget::getConf('Log', 'driver'),
            Gadget::getConf('Log', 'destination'),
            'GADGET'
        );
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     * @param   string  $prefix
     * @param   mixed   $arg
     */
    function execute($prefix, $arg)
    {
        static $source;
        if (!$source) {
            $source = '['. $_SERVER['REMOTE_ADDR'] . $_SERVER['PHP_SELF'] .'] ';
        }

        switch ($prefix)
        {
            case 'widget':
                $message = '['. get_class($arg) .'] '. $arg->template;
            break;
            
            case 'handler':
                if (is_array($arg['action'])) {
                    if (is_string($arg['action'][0])) {
                        $message = implode('::', $arg['action']);
                    } else {
                        $message = get_class($arg['action'][0]) .'->'. $arg['action'][1];
                    }
                } else {
                    $message = $arg['action'];
                }
                for ($p = '', $i = 0, $c = count($arg['params']); $i < $c; $i++) {
                    $p .= var_export($arg['params'][$i], true);
                    if ($i + 1 !== $c) {
                        $p .= ', ';
                    }
                }
                $message .= '('. str_replace("\n", ' ', $p) .')';
            break;
            
            default:
                $message = str_replace("\n", ' ', is_scalar($arg) ? $arg : var_export($arg, true));
        }
        $this->log->setIdent(str_pad(strToUpper($prefix), 10));
        return $this->log->log($source . $message);
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  bool true
     */
    function onSuccess()
    {
        return true;
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        return false;
    }
    
}

?>
