<?php

/**
 * GWDK - Gadget Web Development Kit
 * 
 * Copyright (c) 2004-2005, Michael Wallner, IWORKS
 * All rights reserved.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.11 $
 * 
 * @copyright   2004-2005 Michael Wallner, IWORKS
 * @license     http://dev.iworks.at/gadget/LICENSE.txt GWDK License
 * 
 * @cvs         $Id: Widget.php,v 1.11 2007-03-30 11:40:07 mike Exp $
 */

/**
 * Requires Gadget
 */
require_once 'Gadget.php';

/**
 * Requires Gadget_Object
 */
require_once 'Gadget/Object.php';

/**#@+
 * @ignore
 */
$GLOBALS['_Gadget_Widget']['ident'] = 0;
$GLOBALS['_Gadget_Widget']['queue'] = array();
/**#@-*/

/** 
 * Gadget_Widget
 *
 * Base widget.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Widget
 * @since       1.0
 */
class Gadget_Widget extends Gadget_Object
{
    /**
     * Widget Queue ID
     * 
     * @var int
     */
    var $ID = -1;
    
    /**
     * HTML_QuickForm
     * 
     * @var object
     */
    var $qform;
    
    /**
     * Rendered Form
     * 
     * @var object
     */
    var $form;
    
    /**
     * Current Location
     * 
     * @var string
     */
    var $location;
    
    /**
     * Query String
     * 
     * @var string
     */
    var $queryString;
    
    /**
     * Whether user needs to authenticate
     * 
     * @var bool
     */
    var $auth = false;

    /**
     * Whether user needs special rights
     * 
     * @var mixed
     */
    var $rights = 0;

    /**
     * Whether to run main() at construction time
     * 
     * @var bool
     */
    var $main = true;
    
    /**
     * Whether to run render() at construction time
     * 
     * @var bool
     */
    var $render = false;

    /**
     * Whether to call render() on Gadget::finish()
     * 
     * @var bool
     */
    var $show = true;
    
    /**
     * Whether to call aggregateFormatter() at construction time
     * 
     * @var bool
     */
    var $aggregateFormatter = false;
    
    /**
     * Template
     * 
     * @var string
     */
    var $template;
    
    /**
     * Gadget Formelement Hooks
     * 
     * @var array
     */
    var $elements = array();
    
    /**
     * On Before Construct
     * 
     * @access  protected
     * @return  void
     */
    function onBeforeConstruct()
    {
        // insert into widget queue
        $this->queue();
        
        $this->location = Gadget::location();
        
        // QueryString
        $this->queryString = &$_SERVER['QUERY_STRING'];
    }
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        // log Widget
        Gadget::log('Widget', $this);
        
        // Authentication required? 
        if ($this->auth && !Gadget::currentUser()) {
            Gadget::runService(SERVICE_AUTH);
        }
        
        // Sufficient Rights? 
        if ($this->rights) {
            if (!Gadget::runService(SERVICE_RIGHTS, $this->rights)) {
                $this->unsufficientRights();
            }
        }
        
        // aggregate Formatter
        if ($this->aggregateFormatter) {
            $this->aggregateFormatter();
        }
        
        // call main()
        if ($this->main) {
            $this->main();
        }
        
        // call render()
        if ($this->render) {
            echo $this->render(true);
        }
    }
    
    /**
     * Main
     * 
     * @access  protected
     * @return  void
     */
    function main()
    {
        // void
    }
    
    /**
     * Attach QuickForm
     * 
     * @access  public
     * @return  object
     * @param   string
     * @param   array
     */
    function &form($formName, $attributes = array(), $location = null, $method = 'post')
    {
        isset($location) or $location = $_SERVER['REQUEST_URI'];
        require_once 'HTML/QuickForm.php';
        $this->qform = &new HTML_QuickForm(
            $formName, $method, $location, '_self', $attributes, true);
        $this->qform->setJsWarnings('','');
        return $this->qform;
    }
    
    /**
     * Render
     * 
     * @access  public
     * @return  string
     */
    function render($finish = false)
    {
        $finish && $this->show = false;
        return Gadget::runService(SERVICE_RENDER, $this, true);
    }

    /**
     * Odd Even
     * 
     * @access  public
     * @return  string
     * @param   int
     */
    function oddEven($int, $odd = 'odd', $even = 'even')
    {
        return $int % 2 ? $odd : $even;
    }
    
    /**
     * Load Template
     * 
     * @access  public
     * @return  string
     * @param   string
     * @param   bool
     */
    function load($template, $render = false)
    {
        strpos($template, '.') or $template .= '.html';
        if ($render) {
            $widget = clone($this);
            $widget->template = $template;
            return $widget->render(true);
        } else {
            ob_start();
            readfile(Gadget::getConf('Gadget', 'tpldir') .'/'. $template);
            return ob_get_clean();
        }
    }
    
    /**
     * Aggregate Formatter
     * 
     * @access  protected
     * @return  void
     */
    function aggregateFormatter()
    {
        require_once 'Gadget/Formatter.php';
        if (PHP_VERSION < 5) {
            aggregate($this, 'Gadget_Formatter');
        } elseif (0&&empty($GLOBALS['_Gadget']['Widget']['aggregatedFormatter'])) {
            $GLOBALS['_Gadget']['Widget']['aggregatedFormatter'] = true;
            if (PEAR::loadExtension('classkit')) {
                classkit_aggregate_methods('Gadget_Widget', 'Gadget_Formatter');
            } elseif (PEAR::loadExtension('runkit')) {
                foreach (get_class_methods('Gadget_Formatter') as $method) {
                    runkit_method_copy('Gadget_Widget', $method, 'Gadget_Formatter', $method);
                }
            } else {
                PEAR::raiseError('Could not aggregate Gadget_Formatter. '.
                    'Neither ext/runkit nor ext/classkit are available');
            }
        }
    }function __call($m,$a){return call_user_func_array(array('Gadget_Formatter', $m), $a);}
    
    /**
     * Unsufficient Rights Hook
     * 
     * @access  protected
     * @return  void
     */
    function unsufficientRights()
    {
        Gadget::newWidget(array('template' => 'unsufficientRights.html'));
        $this->render = false;
        $this->show = false;
        $this->main = false;
    }
    
    /**
     * Create a new Widget
     * 
     * @since   1.2
     * @static
     * @see     Gadget::newWidget()
     * @access  public
     * @return  object
     */
    function &create($widget, $options)
    {
        if (is_array($widget)) {
            $options = $widget;
            $widget  = 'Widget';
        }
        
        if ($widget == 'Widget' || $widget == 'Pager') {
                $incfile = 'Gadget/'. $widget .'.php';
                $widget  = 'Gadget_'. $widget;
                require_once $incfile;
        } else {
                $file = strtr($widget, '_', '/');
                require_once 'Widgets/'.$file.'.php';
        }
        $instance = &new $widget($options);
        return $instance;
    }
    
    /**
     * Queue Widget
     * 
     * @since   1.2
     * @access  protected
     * @return  void
     */
    function queue($ID = null)
    {
        $this->ID = isset($ID) ? $ID : $GLOBALS['_Gadget_Widget']['ident']++;
        $GLOBALS['_Gadget_Widget']['queue'][$this->ID] = &$this;
    }
    
    /**
     * Unqueue this Widget
     * 
     * @since   1.2
     * @access  public
     * @return  void
     */
    function unqueue()
    {
        unset($_GLOBALS['_Gadget_Widget']['queue'][$this->ID]);
    }
    
    /**
     * Requeue this Widget
     * 
     * @since   1.2
     * @access  public
     * @return  void
     */
    function requeue()
    {
        $this->unqueue();
        $this->queue();
    }
    
    /**
     * Queue Widget after another
     * 
     * @since   1.2
     * @access  public
     * @return  bool
     * @param   object  Gadget_Widget $after
     */
    function queueAfter(&$after)
    {
        if ($this->ID >= $after->ID) {
            return false;
        }
        
        // get ids of all widgets in the queue
        $IDs = array_keys($GLOBALS['_Gadget_Widget']['queue']);
        
        // append self to the queue if 
        // $after is the last widget in the queue
        if (end($IDs) == $after->ID) {
            $this->requeue();
            return true;
        }
        
        // walk through the queue
        while (null !== $ID = array_shift($IDs)) {
            
            // ignore widgets prior to $this
            if ($ID < $this->ID) {
                continue;
            }
            
            // we found the target ID of the widget
            // insert self at queue postition $ID
            // and exit the loop
            if ($ID > $after->ID) {
                $this->queue($ID);
                break;
            }
            
            // reorder next widget in queue to current position
            $GLOBALS['_Gadget_Widget']['queue'][reset($IDs)]->queue($ID);
        }
        return true;
    }
}

?>
